import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { DATE_FORMAT } from 'app/config/input.constants';
import { IInputData } from '../input-data.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../input-data.test-samples';

import { InputDataService, RestInputData } from './input-data.service';

const requireRestSample: RestInputData = {
  ...sampleWithRequiredData,
  dataDate: sampleWithRequiredData.dataDate?.format(DATE_FORMAT),
  changeDate: sampleWithRequiredData.changeDate?.toJSON(),
  creationDate: sampleWithRequiredData.creationDate?.toJSON(),
};

describe('InputData Service', () => {
  let service: InputDataService;
  let httpMock: HttpTestingController;
  let expectedResult: IInputData | IInputData[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(InputDataService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a InputData', () => {
      const inputData = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(inputData).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a InputData', () => {
      const inputData = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(inputData).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a InputData', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of InputData', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a InputData', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addInputDataToCollectionIfMissing', () => {
      it('should add a InputData to an empty array', () => {
        const inputData: IInputData = sampleWithRequiredData;
        expectedResult = service.addInputDataToCollectionIfMissing([], inputData);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(inputData);
      });

      it('should not add a InputData to an array that contains it', () => {
        const inputData: IInputData = sampleWithRequiredData;
        const inputDataCollection: IInputData[] = [
          {
            ...inputData,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addInputDataToCollectionIfMissing(inputDataCollection, inputData);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a InputData to an array that doesn't contain it", () => {
        const inputData: IInputData = sampleWithRequiredData;
        const inputDataCollection: IInputData[] = [sampleWithPartialData];
        expectedResult = service.addInputDataToCollectionIfMissing(inputDataCollection, inputData);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(inputData);
      });

      it('should add only unique InputData to an array', () => {
        const inputDataArray: IInputData[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const inputDataCollection: IInputData[] = [sampleWithRequiredData];
        expectedResult = service.addInputDataToCollectionIfMissing(inputDataCollection, ...inputDataArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const inputData: IInputData = sampleWithRequiredData;
        const inputData2: IInputData = sampleWithPartialData;
        expectedResult = service.addInputDataToCollectionIfMissing([], inputData, inputData2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(inputData);
        expect(expectedResult).toContain(inputData2);
      });

      it('should accept null and undefined values', () => {
        const inputData: IInputData = sampleWithRequiredData;
        expectedResult = service.addInputDataToCollectionIfMissing([], null, inputData, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(inputData);
      });

      it('should return initial array if no InputData is added', () => {
        const inputDataCollection: IInputData[] = [sampleWithRequiredData];
        expectedResult = service.addInputDataToCollectionIfMissing(inputDataCollection, undefined, null);
        expect(expectedResult).toEqual(inputDataCollection);
      });
    });

    describe('compareInputData', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareInputData(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareInputData(entity1, entity2);
        const compareResult2 = service.compareInputData(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareInputData(entity1, entity2);
        const compareResult2 = service.compareInputData(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareInputData(entity1, entity2);
        const compareResult2 = service.compareInputData(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
