import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IOutputData } from '../output-data.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../output-data.test-samples';

import { OutputDataService } from './output-data.service';

const requireRestSample: IOutputData = {
  ...sampleWithRequiredData,
};

describe('OutputData Service', () => {
  let service: OutputDataService;
  let httpMock: HttpTestingController;
  let expectedResult: IOutputData | IOutputData[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(OutputDataService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a OutputData', () => {
      const outputData = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(outputData).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a OutputData', () => {
      const outputData = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(outputData).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a OutputData', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of OutputData', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a OutputData', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addOutputDataToCollectionIfMissing', () => {
      it('should add a OutputData to an empty array', () => {
        const outputData: IOutputData = sampleWithRequiredData;
        expectedResult = service.addOutputDataToCollectionIfMissing([], outputData);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(outputData);
      });

      it('should not add a OutputData to an array that contains it', () => {
        const outputData: IOutputData = sampleWithRequiredData;
        const outputDataCollection: IOutputData[] = [
          {
            ...outputData,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addOutputDataToCollectionIfMissing(outputDataCollection, outputData);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a OutputData to an array that doesn't contain it", () => {
        const outputData: IOutputData = sampleWithRequiredData;
        const outputDataCollection: IOutputData[] = [sampleWithPartialData];
        expectedResult = service.addOutputDataToCollectionIfMissing(outputDataCollection, outputData);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(outputData);
      });

      it('should add only unique OutputData to an array', () => {
        const outputDataArray: IOutputData[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const outputDataCollection: IOutputData[] = [sampleWithRequiredData];
        expectedResult = service.addOutputDataToCollectionIfMissing(outputDataCollection, ...outputDataArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const outputData: IOutputData = sampleWithRequiredData;
        const outputData2: IOutputData = sampleWithPartialData;
        expectedResult = service.addOutputDataToCollectionIfMissing([], outputData, outputData2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(outputData);
        expect(expectedResult).toContain(outputData2);
      });

      it('should accept null and undefined values', () => {
        const outputData: IOutputData = sampleWithRequiredData;
        expectedResult = service.addOutputDataToCollectionIfMissing([], null, outputData, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(outputData);
      });

      it('should return initial array if no OutputData is added', () => {
        const outputDataCollection: IOutputData[] = [sampleWithRequiredData];
        expectedResult = service.addOutputDataToCollectionIfMissing(outputDataCollection, undefined, null);
        expect(expectedResult).toEqual(outputDataCollection);
      });
    });

    describe('compareOutputData', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareOutputData(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareOutputData(entity1, entity2);
        const compareResult2 = service.compareOutputData(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareOutputData(entity1, entity2);
        const compareResult2 = service.compareOutputData(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareOutputData(entity1, entity2);
        const compareResult2 = service.compareOutputData(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
