import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IUnitConverter } from '../unit-converter.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../unit-converter.test-samples';

import { UnitConverterService } from './unit-converter.service';

const requireRestSample: IUnitConverter = {
  ...sampleWithRequiredData,
};

describe('UnitConverter Service', () => {
  let service: UnitConverterService;
  let httpMock: HttpTestingController;
  let expectedResult: IUnitConverter | IUnitConverter[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(UnitConverterService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a UnitConverter', () => {
      const unitConverter = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(unitConverter).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a UnitConverter', () => {
      const unitConverter = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(unitConverter).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a UnitConverter', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of UnitConverter', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a UnitConverter', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addUnitConverterToCollectionIfMissing', () => {
      it('should add a UnitConverter to an empty array', () => {
        const unitConverter: IUnitConverter = sampleWithRequiredData;
        expectedResult = service.addUnitConverterToCollectionIfMissing([], unitConverter);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(unitConverter);
      });

      it('should not add a UnitConverter to an array that contains it', () => {
        const unitConverter: IUnitConverter = sampleWithRequiredData;
        const unitConverterCollection: IUnitConverter[] = [
          {
            ...unitConverter,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addUnitConverterToCollectionIfMissing(unitConverterCollection, unitConverter);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a UnitConverter to an array that doesn't contain it", () => {
        const unitConverter: IUnitConverter = sampleWithRequiredData;
        const unitConverterCollection: IUnitConverter[] = [sampleWithPartialData];
        expectedResult = service.addUnitConverterToCollectionIfMissing(unitConverterCollection, unitConverter);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(unitConverter);
      });

      it('should add only unique UnitConverter to an array', () => {
        const unitConverterArray: IUnitConverter[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const unitConverterCollection: IUnitConverter[] = [sampleWithRequiredData];
        expectedResult = service.addUnitConverterToCollectionIfMissing(unitConverterCollection, ...unitConverterArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const unitConverter: IUnitConverter = sampleWithRequiredData;
        const unitConverter2: IUnitConverter = sampleWithPartialData;
        expectedResult = service.addUnitConverterToCollectionIfMissing([], unitConverter, unitConverter2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(unitConverter);
        expect(expectedResult).toContain(unitConverter2);
      });

      it('should accept null and undefined values', () => {
        const unitConverter: IUnitConverter = sampleWithRequiredData;
        expectedResult = service.addUnitConverterToCollectionIfMissing([], null, unitConverter, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(unitConverter);
      });

      it('should return initial array if no UnitConverter is added', () => {
        const unitConverterCollection: IUnitConverter[] = [sampleWithRequiredData];
        expectedResult = service.addUnitConverterToCollectionIfMissing(unitConverterCollection, undefined, null);
        expect(expectedResult).toEqual(unitConverterCollection);
      });
    });

    describe('compareUnitConverter', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareUnitConverter(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareUnitConverter(entity1, entity2);
        const compareResult2 = service.compareUnitConverter(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareUnitConverter(entity1, entity2);
        const compareResult2 = service.compareUnitConverter(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareUnitConverter(entity1, entity2);
        const compareResult2 = service.compareUnitConverter(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
